#include "xparameters.h"
#include "xgpio_l.h"
#include "xbasic_types.h"
#include "sleep.h"


#define FLASH_DATA (*(unsigned*)FLASH_ADDRESS)
unsigned FLASH_ADDRESS;

extern int fVerbose;

Xuint32 str2Xuint32(const char *p, char **out_p, int base)
{
	Xint32 v = 0;

	while (isspace(*p))
		p++;
	
	if (((base == 16) || (base == 0)) && ((*p == '0') && ((p[1] == 'x') || (p[1] == 'X'))))	{
		p += 2;
		base = 16;
	}
	
	if (base == 0) {
		if (*p == '0')
			base = 8;
		else
			base = 10;
	}
	
	while (1) {
		char c = *p;
		if ((c >= '0') && (c <= '9') && (c - '0' < base))
			v = (v * base) + (c - '0');
		else if ((c >= 'a') && (c <= 'z') && (c - 'a' + 10 < base))
			v = (v * base) + (c - 'a' + 10);
		else if ((c >= 'A') && (c <= 'Z') && (c - 'A' + 10 < base))
			v = (v * base) + (c - 'A' + 10);
		else
			break;
		p++;
	}

	if (out_p) 
		*out_p = (char*)p;
	return v;
}

///////////////////////////////////////////////
//  Flash_Reset                              //
//  Reset the flash and put it in Read mode  //
///////////////////////////////////////////////
void Flash_Reset_Read(unsigned base_addr){
/* Reset to read mode */
	FLASH_ADDRESS = base_addr;
	FLASH_DATA = 0x00f000f0;
	usleep(20);
}


/////////////////////////////////////////////
//  Flash_Autoselect                       //
//  Put the flash in "Autoselect" mode     //
/////////////////////////////////////////////
void Flash_Autoselect(unsigned base_addr)
{
	unsigned temp;
	unsigned x;

/* Place device in Auto Select mode  */
	FLASH_ADDRESS = base_addr + (0x555 << 2);
	/* 1st Cycle   */
	FLASH_DATA = 0x00AA00AA;
	usleep(2); //delay made with usleep()
	
	FLASH_ADDRESS = base_addr + (0x2AA << 2);
	/* 2nd Cycle   */
	FLASH_DATA = 0x00550055;
	usleep(2);
	
	FLASH_ADDRESS = base_addr + (0x555 << 2);
	/* 3rd Cycle   */
	FLASH_DATA = 0x00900090;
	usleep(2);
}

/////////////////////////////////////////////
//  Flash_Write                            //
/////////////////////////////////////////////
unsigned Flash_Write(unsigned base_addr, unsigned prog_addr, unsigned prog_data)
//returns 
//0 if flash has been correctly programmed
//1 if address is not multiple of 4
//2 if address is too high
//3 if flash write error

{
	unsigned temp, temp1;
	unsigned x;
	unsigned FLASH_ADRESS_LIST[3]={	0x555, //program  mode command adresses
												0x2AA, //for the flash memory
												0x555};//look into the datasheet for details
												
	unsigned FLASH_DATA_LIST[3 ]={	0x00AA00AA,//program command data
												0x00550055,
												0x00A000A0};
	unsigned i;	
	
	
	if (prog_addr&0x03) return 1; //address is not multiple of 4
	if (prog_addr>0xFFFFFC) return 2; //address too high


//	Reset to read mode
	Flash_Reset_Read(base_addr);

	for(i=0; i<3; i++) //send program command
	{
		FLASH_ADDRESS = base_addr + (FLASH_ADRESS_LIST[i] << 2);
		FLASH_DATA = FLASH_DATA_LIST[i];
		usleep(1);
	}

//	program address and data

	FLASH_ADDRESS = base_addr + (prog_addr);
	FLASH_DATA = prog_data;

	//READING STATUS
	temp1=0;
	temp=FLASH_DATA;
	usleep(1);
	
	while ((((temp&0x00800000)^(prog_data&0x00800000))||((temp&0x00000080)^(prog_data&0x00000080))))//test bit DQ7
	{
			temp=FLASH_DATA;
			//test bit DQ5 and DQ7 on the upper memory to see if some error ocurred
			if ((temp&0x00200000)&&(!((temp&0x00800000)^(prog_data&0x00800000)))) //DQ(5+16)==1, DQ(7+16)== NOT DQ(7+16)_prog_data - complement
			{
				if ((temp&0x00000020)&&(!((temp&0x00000080)^(prog_data&0x00000080)))) return 0; //DQ5==1, DQ(7+16) no complement
				//memory program ended sucessfully
			}
			else if (((temp&0x00200000)&&((temp&0x00800000)^(prog_data&0x00800000)))) //DQ(5+16)==1, DQ(7+16) complement 
				{//upper memory error
				Flash_Reset_Read(base_addr);
				return 1; 
				}
			//test bit DQ5 and DQ7 on the lower memory to see if some error ocurred
			if ((temp&0x00000020)&&(!((temp&0x00000080)^(prog_data&0x00000080)))) //DQ5==1, DQ7 no complement
			{
				if ((temp&0x00200000)&&(!((temp&0x00000080)^(prog_data&0x00000080)))) return 0; //DQ(5+16)==1, DQ(7+16) no complement,
				//memory program ended sucessfully
			}
			else if ((temp&0x00000020)&&((temp&0x00000080)^(prog_data&0x00000080)))//DQ5==1, DQ7 complement
				{//lower memory error
				Flash_Reset_Read(base_addr);
				return 2;
				}
		usleep(1);
	}
	return 0;


}

/////////////////////////////////////////////
//  Flash_Block_Erase                       //
/////////////////////////////////////////////
unsigned Flash_Block_Erase(unsigned base_addr, int *block_num)
{
//returns 
//0 if flash has been correctly erased
//1 if upper memory erase error
//2 if lower memory erase error

	unsigned temp, temp1;
	unsigned FLASH_ADRESS_LIST[5]={	0x555, //block erase mode command adresses
												0x2AA, //for the flash memory
												0x555,//look into the datasheet
												0x555,//for details
												0x2AA};
												
	unsigned FLASH_DATA_LIST[5 ]={	0x00AA00AA,//block erase command data
												0x00550055,
												0x00800080,
												0x00AA00AA,
												0x00550055};
	int i;

	
/* Reset to read mode */
	Flash_Reset_Read(base_addr);

	for(i=0; i<5; i++) //send the block erase command
	{
		FLASH_ADDRESS = base_addr + (FLASH_ADRESS_LIST[i] << 2);
		FLASH_DATA = FLASH_DATA_LIST[i];
		usleep(1);
	}

	while (*block_num >=0)//send the block adressses
	{
		i = *block_num;
		if (*block_num<=8) temp = (i << 12)<<2 ; //calculate the corresponding block adrress
		else temp = ((i - 7 ) << 15) <<2;
		FLASH_ADDRESS = base_addr + temp;
		FLASH_DATA = 0x00300030;//send 0x0030 to each memory block (here are two memory chips)		
		usleep(1);
		block_num++;
	}

	FLASH_ADDRESS = base_addr;
	temp=FLASH_DATA; //check DQ3 to see if erasing has been started
	usleep(1);	
	
	while (!((temp&0x00080000)&&(temp&0x00000008)))
	{
		temp=FLASH_DATA;
		usleep(1);
	}
	//READING STATUS
	temp1=0;
	temp=FLASH_DATA;
	usleep(1);
	while (!((temp&0x00800000)&&(temp&0x00000080)))//test bit DQ7
	{
			temp=FLASH_DATA;
			//test bit DQ5 and DQ7 on the upper memory to see if some error ocurred
			if ((temp&0x00200000)&&(temp&0x00800000)) //DQ(5+16)==1, DQ(7+16)==1
			{
				if ((temp&0x00000020)&&(temp&0x00000080)) return 0; //DQ5==1, DQ7==1,
				//memory erase ended sucessfully
			}
			else if ((temp&0x00200000)&&!(temp&0x00800000)) //DQ(5+16)==1, DQ(7+16)==0
				{//upper memory error
				Flash_Reset_Read(base_addr);
				return 1; 
				}
			//test bit DQ5 and DQ7 on the lower memory to see if some error ocurred
			if ((temp&0x00000020)&&(temp&0x00000080)) //DQ5==1, DQ7==1
			{
				if ((temp&0x00200000)&&(temp&0x00800000)) return 0; //DQ(5+16)==1, DQ(7+16)==1,
				//memory erase ended sucessfully
			}
			else if ((temp&0x00000020)&&!(temp&0x00000080))//DQ5==1, DQ7==0
				{//lower memory error
				Flash_Reset_Read(base_addr);
				return 2;
				}
		usleep(1);

		if (fVerbose)
		{
			temp1++;
			if(!(temp1&0x0000FFFF))//print a dot periodically
			print(".");
		}
	}
	return 0;
}


